%% Copyright (c) 2012 Miguel Bazdresch
%%
%% Permission is hereby granted, free of charge, to any person obtaining a
%% copy of this software and associated documentation files (the "Software"),
%% to deal in the Software without restriction, including without limitation
%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
%% and/or sell copies of the Software, and to permit persons to whom the
%% Software is furnished to do so, subject to the following conditions:
%%
%% The above copyright notice and this permission notice shall be included in
%% all copies or substantial portions of the Software.
%%
%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
%% DEALINGS IN THE SOFTWARE.

%% usage: [ssf axfs pfxs] = dft(x [, Ts [, N]])
%%
%% Calculate the discrete Fourier transform (DFT) of a signal. This
%% function replaces fft(); the idea is to make the calculations more
%% transparent for didactic purposes. It _is_ slower than fft(), but for
%% classroom examples on a modern machine it shouldn't make much difference.
%%
%% Input arguments:
%%
%% x : signal (complex or real)
%% Ts : sampling period (real > 0), default is 0.5.
%% N  : in case zero-padding is desired. If numel(x)<N, then x is padded with
%%      enough zeros so that numel(x)=N.
%%
%% Output:
%%
%% ssf : vector of frequencies
%% afxs : DFT magnitude
%% pfxs : DFT phase

%% Author: Miguel Bazdresch

function [ssf afxs pfxs] = dft(x, Ts=0.5, N=0)

	fs = 1/Ts;

	if( nargin == 0 )
	    usage( "At least one argument is needed" );
	end

	% zero padding
	nx = numel(x);
	if nx < N
		x = [ x zeros(1,N-nx) ];
	end
	N = numel(x)

	% preallocate complex vector
	DFT = complex(zeros(1,N),zeros(1,N));
	% if x is real, reduce calculations using symmetry
	if isreal(x)
		L = floor(N/2);
		for k=0:L
			DFT(k+1) = sum(x.*exp(-j*2*pi*k*(0:N-1)/N));
		end
		for k=L+1:N-1
			DFT(k+1) = conj(DFT(N-k+1));
		end
	else
		for k=0:N
			DFT(k+1) = sum(x.*exp(-j*2*pi*k*(0:N-1)/N));
		end
	end
	
	DFT = [ DFT((floor((N-1)/2)+2):N) DFT(1) DFT(2:(floor((N-1)/2)+1)) ];
	afxs = abs(DFT);
	pfxs = unwrap(angle(DFT));
	df = fs/N;
	ssf = [ -(ceil((N-1)/2):-1:1)*df 0 (1:floor((N-1)/2))*df ];

end
