%% Copyright (c) 2012 Miguel Bazdresch
%%
%% Permission is hereby granted, free of charge, to any person obtaining a
%% copy of this software and associated documentation files (the "Software"),
%% to deal in the Software without restriction, including without limitation
%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
%% and/or sell copies of the Software, and to permit persons to whom the
%% Software is furnished to do so, subject to the following conditions:
%%
%% The above copyright notice and this permission notice shall be included in
%% all copies or substantial portions of the Software.
%%
%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
%% DEALINGS IN THE SOFTWARE.

%% usage: [dens f en pot] = psd( s [, Ts=0.5 [, pad=1]] )
%%
%% Calculate the power spectral density (PSD) of the periodic exansion of
%% a non-random, finite signal
%%
%% This function estimates the PSD of an analog signal that is represented by
%% a vector of samples. From the PSD, the signal's average power and energy
%% are calculated. By Parseval, these are the same quantites that would be
%% found if calculated in the time domain.
%%
%% Input arguments:
%%
%% x : signal (complex or real)
%% Ts : sampling period (real > 0), default is 0.5.
%% pad : if for some reason the signal shouldn't be padded with zeros, set
%%  pad to 0.
%%
%% Output:
%%
%% f : a vector of frequencies
%% dens : the PSD
%% en : the signal's energy calculated in the frequency domain
%% pot : the average signal's power during the signal's duration, calculated
%%       in the frequency domain
%%
%% Example:
%%
%% Ts = 0.001;
%% t = 0:Ts:2;
%% x = sin(2*pi*200*t);
%% [f,dens,en,pot]=esd(x,Ts);
%% plot(f,dens);
%%
%% reference: http://www.mathworks.com/support/tech-notes/1700/1702.html

%% Author: Miguel Bazdresch

function [f dens en pot] = psd( s, Ts=0.5, pad=1 )

	if nargin < 1 then
		usage('At least one argument is required.');
	end

	ls = length(s);
	Tx = ls*Ts;  % signal duration before padding
	if pad == 1
		N = 2^(nextpow2(ls)+1);
	else
		N = ls;
	end
	T = N*Ts;  % signal duration after padding
	df = 1/T;  % frequency sampling interval
	dens = ((1/ls)*abs(fft(s,N))).^2;
	dens = fftshift(dens);
	f = (-N/2:N/2-1)*df;  % frequency vector
	pot = Tx*trapz(f,dens);
	en = Tx*pot;

end
