%% Copyright (c) 2012 Miguel Bazdresch
%%
%% Permission is hereby granted, free of charge, to any person obtaining a
%% copy of this software and associated documentation files (the "Software"),
%% to deal in the Software without restriction, including without limitation
%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
%% and/or sell copies of the Software, and to permit persons to whom the
%% Software is furnished to do so, subject to the following conditions:
%%
%% The above copyright notice and this permission notice shall be included in
%% all copies or substantial portions of the Software.
%%
%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
%% DEALINGS IN THE SOFTWARE.

%% usage: [p t] = rcpulse(beta, D, Tp, Ts [, type [, energy]])
%%
%% Create sinc, raised cosine and square-root raised cosine pulses.
%%
%% Input arguments:
%%
%%    beta : rolloff factor
%%    D : pulse duration as a multiple of symbol periods (must be even)
%%    Tp : symbol period (time between symbols; reciprocal of symbol rate)
%%    Ts : sampling period (Ts must divide Tp)
%%    type : (string) pulse type
%%        'srrc' - square-root raised cosine
%%        'rc'   - raised cosine (default)
%%    energy : pulse energy (1 by default)
%%
%% output:
%%
%%    p : pulse samples
%%    t : correspoding time vector, starting at t=0
%%
%% Example:
%% Generates an SRRC pulse with Ts=1/8000, bandwith = 1000Hz, that
%% allows a symbol rate of 2000 pulses per second, with a duration of ten
%% symbol periods (1/200 seconds) and energy equal to 0.5 joules:
%%
%% [p t] = rcpulse(0,10,1/2000,1/8000,'srrc',0.5);
%%
%% references:
%%  Telecommunication Breakdown (by Johnson and Sethares), pages 217 and 225
%%  http://www.dsplog.com/2008/04/22/raised-cosine-filter-for-transmit-pulse-shaping/
%%  http://en.wikipedia.org/wiki/Raised-cosine_filter
%%  http://en.wikipedia.org/wiki/Root-raised-cosine_filter

%% Author:  Luis Miguel Bazdresch Sierra

function [p t] = rcpulse(beta, D, Tp, Ts, type, energy)

	if nargin < 4
		usage('At least four arguments are required.');
	end
	if nargin < 6
		energy = 1;
	end
	if nargin == 4
		type = 'rc';
	end

	%% D must be even
	if mod(D,2) == 1
		usage('D must be even.');
	end

	%% Tp must be (close to) an integer multiple of Ts
	if abs(round(Tp/Ts) - Tp/Ts) > 1e-3
		usage('Tp must be an integer multiple of Ts');
	end

	%% avoid division by zero warnings
	if beta==0
		beta=1e-12;
	end;

	%% non causal time vector used to build the pulse, so that the pulse is
	%% symmetric (even).
	t = -D*Tp/2:Ts:D*Tp/2;

	if strcmp(type, "srrc")
		%% square-root raised cosine (if beta==0, returns a sinc pulse)
		%% calculation is done in three parts: negative time, t=0, and
		%% postive time
		t1 = -D*Tp/2:Ts:-Ts;
		t2 = Ts:Ts:D*Tp/2;

		x1 = pi*(1-beta)*t1/Tp;
		x2 = pi*(1+beta)*t1/Tp;
		x3 = 4*beta*t1/Tp;
		num = sin(x1)+x3.*cos(x2);
		den = sqrt(Tp).*(pi*t1./Tp).*(1-x3.^2);
		p1 = num./den;
		%% break long calculation
		_t1 = (beta/sqrt(2*Tp));
		_t2 = (1+2/pi)*sin(pi/(4*beta));
		p1(find(abs(den)<1e-9))=_t1*(_t2+(1-2/pi)*cos(pi/(4*beta)));

		p2 = 1/sqrt(Tp)*(1-beta+4*beta/pi);

		x1 = pi*(1-beta)*t2/Tp;
		x2 = pi*(1+beta)*t2/Tp;
		x3 = 4*beta*t2/Tp;
		num = sin(x1)+x3.*cos(x2);
		den = sqrt(Tp).*(pi*t2./Tp).*(1-x3.^2);
		p3 = num./den;
		_t1 = (beta/sqrt(2*Tp));
		_t2 = (1+2/pi)*sin(pi/(4*beta));
		p3(find(abs(den)<1e-9))=_t1*(_t2+(1-2/pi)*cos(pi/(4*beta)));

		p = [p1 p2 p3];

	elseif strcmp(type, "rc")
		%% raised cosine (if beta==0, returns a sinc pulse)
		den = 1-(2*beta*t/Tp).^2;
		c = cos(pi*beta*t/Tp)./den;
		%% special case: where the denominator becomes zero
		c(find(abs(den)<1e-9)) = pi/4;
		p = (1/Tp).*sinc(t/Tp).*c;
	end

	%% return a causal pulse
	t = t + D*Tp/2;

	%% energy normalization
	en = trapz(t,p.*p);
	p = sqrt(energy/en)*p;

end
