%% Copyright (c) 2012 Miguel Bazdresch
%%
%% Permission is hereby granted, free of charge, to any person obtaining a
%% copy of this software and associated documentation files (the "Software"),
%% to deal in the Software without restriction, including without limitation
%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
%% and/or sell copies of the Software, and to permit persons to whom the
%% Software is furnished to do so, subject to the following conditions:
%%
%% The above copyright notice and this permission notice shall be included in
%% all copies or substantial portions of the Software.
%%
%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
%% DEALINGS IN THE SOFTWARE.

%% usage: specgr(filename [, t_slice [, t_step [, clip_min [, clip_max
%%   [, cmap [, ftitle]]]]])
%%
%% A wrapper function for Octave's signal package's specgram function, which
%% takes its input from a WAV file. Only the WAV file's first channel is
%% analyized; all other channels are discarded.
%%
%% Input arguments:
%%
%% filename : (string) a WAV-format filename
%% t_slice  : spectogram slice time (in milliseconds, default 10ms)
%% t_step   : time between slices (in ms, default 5ms)
%% clip_min : clip values below clip_min to clip_min (in dB, default -60)
%% clip_max : clip values above clip_max to clip_max (in dB, default 0dB). Only
%%            values below 0dB make sense (the signal is normalized so that
%%            its maximum value is equal to 1).
%% cmap     : (string) colormap, default 'jet'
%% ftitle   : (string) figure title, default 'Spectrogram'

%% Author: Miguel Bazdresch

function specgr(filename, t_slice, t_step, clip_min, clip_max, cmap, ftitle)

	if nargin == 0
		usage('At least one argument is required.');
	end
	if nargin < 7
		ftitle = 'Spectrogram';
	end
	if nargin < 6
		cmap = 'jet';
	end
	if nargin < 5
		clip_max = 0;
	end
	if nargin < 4
		clip_min = -60;
	end
	if nargin < 3
		t_step = 5;
	end
	if nargin < 2
		t_slice = 10;
	end

	% read one channel of WAV file
	[y,Fs] = wavread(filename);
	x = y(:,1)';
	clear y; % don't keep in memory variables we don't need

	% required parameters
	step = fix(t_step*Fs/1000);     % spectral slices
	window = fix(t_slice*Fs/1000);  % data window
	fftn = 2^nextpow2(window);      % pad fft
	[S, f, t] = specgram(x, fftn, Fs, window, window-step);

	% plot
	figure();
	S = abs(S);
	S = S/max(S(:));
	S = max(S, 10^(clip_min/10));    % clip below clip_min
	S = min(S, 10^(clip_max/10));    % clip above clip_max
	imagesc(t, f, log10(S));         % log scale
	set (gca(), "ydir", "normal");   % frequency increases in the y direction
	colormap(feval(cmap));           % cmap is a function (see doc colormap)
	title(ftitle);
	xlabel('time');
	ylabel('frequency');

end
