%% Copyright (c) 2012 Miguel Bazdresch
%%
%% Permission is hereby granted, free of charge, to any person obtaining a
%% copy of this software and associated documentation files (the "Software"),
%% to deal in the Software without restriction, including without limitation
%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
%% and/or sell copies of the Software, and to permit persons to whom the
%% Software is furnished to do so, subject to the following conditions:
%%
%% The above copyright notice and this permission notice shall be included in
%% all copies or substantial portions of the Software.
%%
%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
%% DEALINGS IN THE SOFTWARE.

%% usage: [ssf afxs pfxs] =
%%        spectrum(x [, Ts [, N [, win [, dp [, clip [, pctrl]]]]]])
%%
%% Find the spectrum of a signal, using the Fast Fourier Transform. Originally
%% based on 'plotspec.m' from the book Telecommunications Breakdown, by Johnson
%% and Sothares, this function allows control over many of the fft and plotting
%% parameters, and adds phase spectrum plotting.
%%
%% Input Arguments:
%%
%% x  : signal
%% Ts : sampling period (real > 0), default = 0.5
%% N  : fft size (integer)
%%      0 : (default) N = 2^nextpow2(length(x))
%%      y : N = y
%% win : window input data (0 - 1)
%%      0 : (default) no windowing
%%      1 : hanning window
%% dp : control plotting (0 - 4)
%%      0 : (default) no plotting
%%      1 : plot mag spec and phase spec in two figures
%%      2 : plot mag spec only
%%      3 : plot phase spec only
%%      4 : plot mag spec and phase spec in one figure
%% clip : clip amplitudes before finding phase (0 - 100)
%%      0 : (default) no clipping
%%      y : clip frequency to zero if amplitude at that frequency is less
%%          than y
%% pctrl : which command to plot with
%%      0 : (default) use stem
%%      1 : use plot
%%      2 : use semilogy
%%
%% Output:
%%
%% ssf  : vector of frequencies
%% afxs : magnitude spectrum
%% pfxs : phase spectrum
%%
%% Example:
%%
%% Ts = 1/1000;
%% t = 0:Ts:1;
%% x = 0.5*cos(2*pi*100*t)+0.2*sin(2*pi*200*t)+2*cos(2*pi*300*t);
%% spectrum(x,Ts,4096,1,4,0.03);

%% Author: Luis Miguel Bazdresch Sierra

function [ssf afxs pfxs] = spectrum(x, Ts, N, win, dp, clip, pctrl)

	if( nargin == 0 )
		usage( 'At least one argument is needed' );
	end
	if( nargin < 7 )
		pctrl = 0;
	end
	if( nargin < 6 )
		clip = 0;
	end
	if( nargin < 5 )
		dp = 0;
	end
	if( nargin < 4 )
		win = 0;
	end
	if( nargin < 3 )
		N = 0;
	end
	if( nargin < 2 )
		Ts = 0.5;
	end

	if( N == 0 )
		N = 2^nextpow2(length(x));
	end

	ssf=(-N/2:N/2-1)/(N*Ts);              % frequency vector
	if( win == 1 )
		x = hanning(length(x))'.*x;       % hanning window
	end
	fx = fftshift(fft(x,N));              % FFT
	afxs = abs(fx)/length(x);             % magnitude spectrum (scaled)
	pfxs = angle(fx);                     % phase spectrum
	if( clip ~= 0 )                       % clip
		pfxs(afxs<clip) = 0;
	end
	%pfxs = unwrap(pfxs);
	% plots
	if( dp == 1 || dp == 2 || dp == 4 )
		figure();
		if( dp == 4 )
			subplot(2,1,1);
		end
		if( pctrl == 0)
			stem(ssf,afxs);
		elseif( pctrl == 1 )
			plot(ssf,afxs);
		elseif( pctrl == 2 )
			semilogy(ssf,afxs);
		end
		title('Magnitude spectrum');
		xlabel('Frequency (Hz)');
		ylabel('Amplitude');
	end
	if( dp == 1 || dp == 3 || dp == 4)
		if( dp == 4 )
			subplot(2,1,2);
		else
			figure();
		end
		if( pctrl == 0)
			stem(ssf,pfxs);
		elseif( pctrl == 1 )
			plot(ssf,pfxs);
		elseif( pctrl == 2 )
			semilogy(ssf,pfxs);
		end
		title('Phase spectrum');
		xlabel('Frequency (Hz)');
		ylabel('Phase (radians)');
	end

end
